package com.hcordova.flightagencyapi.flight.services;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.List;

import org.springframework.stereotype.Service;
import org.springframework.web.bind.annotation.ResponseBody;

import com.hcordova.flightagencyapi.aircraft.domain.services.AircraftService;
import com.hcordova.flightagencyapi.airline.domain.services.AirlineService;
import com.hcordova.flightagencyapi.airport.domain.models.Airport;
import com.hcordova.flightagencyapi.airport.domain.services.AirportService;
import com.hcordova.flightagencyapi.flight.domain.models.FlightCost;
import com.hcordova.flightagencyapi.flight.domain.models.FlightSchedule;
import com.hcordova.flightagencyapi.flight.domain.models.Leg;
import com.hcordova.flightagencyapi.flight.domain.repositories.FlightScheduleRepository;
import com.hcordova.flightagencyapi.flight.domain.services.FlightScheduleService;
import com.hcordova.flightagencyapi.flight.dto.FlightSaveDTO;
import com.hcordova.flightagencyapi.shared.exceptions.NotFoundException;

import lombok.RequiredArgsConstructor;

@Service
@RequiredArgsConstructor
public class FlightScheduleServiceImpl implements FlightScheduleService{
    
    private final FlightScheduleRepository flightScheduleRepository;
    private final AirlineService airlineService;
    private final AircraftService aircraftService;
    private final AirportService airportService;

    @Override
    public List<FlightSchedule> getAll() {
        return flightScheduleRepository.findAllByDeletedAtIsNull();
    }

    @Override
    public FlightSchedule save(FlightSaveDTO saveDTO) {

        FlightSchedule flightSchedule = new FlightSchedule();

        // Set airline and aircraft
        flightSchedule.setAirline(airlineService.getById(saveDTO.getAirlineId()));
        flightSchedule.setAircraft(aircraftService.getById(saveDTO.getAircraftId()));

        // Valid legs size
        if (saveDTO.getLegs().size() == 0) 
            throw new RuntimeException("Invalid legs size");

        if (saveDTO.getCosts().size() == 0) 
            throw new RuntimeException("Invalid costs size");

        List<Leg> legsToSave = saveDTO.getLegs().stream().map(leg -> {
            Airport airportOrigin = airportService.getById(leg.getAirportOriginId());
            Airport airportDestination = airportService.getById(leg.getAirportDestinationId());
            try {
                LocalDateTime departureDateTime = LocalDateTime.parse(leg.getDepartureDateTime());
                LocalDateTime arrivalDateTime = LocalDateTime.parse(leg.getArrivalDateTime());
                return new Leg(null, flightSchedule, airportOrigin, airportDestination, departureDateTime, arrivalDateTime, null);
            } catch (Exception e) {
                throw new RuntimeException("Invalid date format 1");
            }
        }).toList();

        List<FlightCost> costsToSave = saveDTO.getCosts().stream().map(cost -> {
            try {
                LocalDate validFrontDate = LocalDate.parse(cost.getDateStart());
                LocalDate validToDate = LocalDate.parse(cost.getDateEnd());
                return new FlightCost(null, flightSchedule, validFrontDate, validToDate, cost.getCost());
            } catch (Exception e) {
                throw new RuntimeException("Invalid date format 2");
            }
        }).toList();

        // Set departure and arrival date
        try {
            LocalDateTime departureDate = LocalDateTime.of(LocalDate.parse(saveDTO.getDepartureDate()), legsToSave.get(0).getActualDeparture().toLocalTime());
            LocalDateTime arrivalDate = LocalDateTime.of(LocalDate.parse(saveDTO.getArrivalDate()), legsToSave.get(legsToSave.size() - 1).getActualArrival().toLocalTime());
            flightSchedule.setDepartureDateTime(departureDate);
            flightSchedule.setArrivalDateTime(arrivalDate);
        } catch (Exception e) {
            throw new RuntimeException("Invalid date format 3");
        }

        // Set airport origin and destination
        flightSchedule.setAirportOrigin(legsToSave.get(0).getAirportOrigin());
        flightSchedule.setAirportDestination(legsToSave.get(legsToSave.size() - 1).getAirportDestination());

        // Save legs and costs
        flightSchedule.setLegs(legsToSave);
        flightSchedule.setCosts(costsToSave);

        return flightScheduleRepository.save(flightSchedule);
    }

    @Override
    public FlightSchedule getById(Long id) {
        return flightScheduleRepository.findByIdAndDeletedAtIsNull(id)
            .orElseThrow(() -> new NotFoundException(String.format("Flight with id %d not found", id)));
    }

    @Override
    public FlightSchedule delete(Long id) {
        FlightSchedule flightSchedule = getById(id);
        flightSchedule.setDeletedAt(LocalDate.now());
        return flightScheduleRepository.save(flightSchedule);
    }

    @Override
    @ResponseBody
    public List<FlightSchedule> searchByOriginDestinationAndDate(Long airportOriginId, Long airportDestinationId,
            String startDate) {
        LocalDate date = LocalDate.parse(startDate);
        LocalDateTime dateTime = date.atStartOfDay();
        return flightScheduleRepository.searchFlight(airportOriginId, airportDestinationId, dateTime);
    }
}
