package com.hcordova.flightagencyapi.reservation.services;

import java.time.LocalDate;
import java.util.List;

import org.springframework.stereotype.Service;

import com.hcordova.flightagencyapi.agent.domain.services.AgentService;
import com.hcordova.flightagencyapi.flight.domain.services.FlightScheduleService;
import com.hcordova.flightagencyapi.passenger.domain.services.PassengerService;
import com.hcordova.flightagencyapi.payment.domain.models.PaymentStatus;
import com.hcordova.flightagencyapi.payment.domain.services.PaymentStatusService;
import com.hcordova.flightagencyapi.reservation.domain.models.Payment;
import com.hcordova.flightagencyapi.reservation.domain.models.Reservation;
import com.hcordova.flightagencyapi.reservation.domain.repositories.ReservationsRepository;
import com.hcordova.flightagencyapi.reservation.domain.services.ReservationService;
import com.hcordova.flightagencyapi.reservation.domain.services.ReservationStatusService;
import com.hcordova.flightagencyapi.reservation.domain.services.TicketTypeService;
import com.hcordova.flightagencyapi.reservation.domain.services.TravelClassService;
import com.hcordova.flightagencyapi.reservation.dto.ReservationSaveDTO;
import com.hcordova.flightagencyapi.shared.exceptions.NotFoundException;

import jakarta.transaction.Transactional;
import lombok.RequiredArgsConstructor;

@Service
@RequiredArgsConstructor
public class ReservationServiceImpl implements ReservationService {

    private final ReservationsRepository reservationsRepository;
    private final TicketTypeService ticketTypeService;
    private final TravelClassService travelClassService;
    private final AgentService agentService;
    private final PassengerService passengerService;
    private final FlightScheduleService flightScheduleService;
    private final ReservationStatusService reservationStatusService;
    private final PaymentStatusService paymentStatusService;
    
    @Override
    public List<Reservation> getAll() {
        return reservationsRepository.findByDeletedAtIsNull();
    }

    @Override
    public Reservation getById(Long id) {
        return reservationsRepository.findByIdAndDeletedAtIsNull(id)
            .orElseThrow(() -> new NotFoundException(String.format("Reservation with id %d not found", id)));
    }

    @Override
    @Transactional
    public Reservation save(ReservationSaveDTO reservationDTO) {

        Reservation reservation = new Reservation();

        var ticketType = ticketTypeService.getById(reservationDTO.getTicketTypeId());
        var travelClass = travelClassService.getById(reservationDTO.getTravelClassId());
        var passenger = passengerService.getById(reservationDTO.getPassengerId());
        var agent = agentService.getById(reservationDTO.getAgentId());
        var legs = flightScheduleService.getById(reservationDTO.getFlightId()).getLegs();
        var status = reservationStatusService.findByName("registered");

        legs.stream().map(leg -> leg.getReservations().add(reservation));
        reservation.getLegs().addAll(legs);

        try {
            reservation.setCreatedAt(LocalDate.parse(reservationDTO.getReservationMade()));
        } catch (Exception e) {
            throw new RuntimeException("Invalid date format");
        }

        Payment payment = new Payment();
        PaymentStatus paymentStatus = paymentStatusService.findByName("pending");

        payment.setDate(LocalDate.now());
        payment.setStatus(paymentStatus);
        payment.setAmount(reservationDTO.getCost());

        reservation.setAgent(agent);
        reservation.setPassenger(passenger);
        reservation.setTicketType(ticketType);
        reservation.setTravelClass(travelClass);
        reservation.setStatus(status);
        reservation.setPayment(payment);

        return reservationsRepository.save(reservation);
    }

    @Override
    public List<Reservation> getByPassengerId(Long id) {
        return reservationsRepository.findByDeletedAtIsNullAndPassenger_Id(id);
    }

    @Override
    public Reservation payReservation(Long id) {
        Reservation reservation = getById(id);
        PaymentStatus paymentStatus = paymentStatusService.findByName("paid");
        reservation.getPayment().setStatus(paymentStatus);
        reservation.setStatus(reservationStatusService.findByName("paid"));
        return reservationsRepository.save(reservation);
    }

    @Override
    public Reservation cancelReservation(Long id) {
        Reservation reservation = getById(id);

        System.out.println(reservation.getStatus().getName().toLowerCase());
        
        
        if (reservation.getStatus().getName().toLowerCase().equals("registered")) {
            PaymentStatus paymentStatus = paymentStatusService.findByName("refunded");
            reservation.getPayment().setStatus(paymentStatus);
            reservation.setStatus(reservationStatusService.findByName("Cancelled"));
            return reservationsRepository.save(reservation);
        } else {
            System.out.println(id);
            throw new RuntimeException("Reservation already paid");
        }
    }

    @Override
    public Reservation deleteReservation(Long id) {
        Reservation reservation = getById(id);
        if (reservation.getStatus().getName().toLowerCase().equals("registered") ||
            reservation.getStatus().getName().toLowerCase().equals("cancelled")) {
            reservation.setDeletedAt(LocalDate.now());
            return reservationsRepository.save(reservation);
        } else {
            throw new RuntimeException("Reservation already paid");
        }
        
    }
}
