package com.hcordova.flightagencyapi.shared.services;

import java.time.LocalDate;

import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;

import com.hcordova.flightagencyapi.agent.domain.models.Agent;
import com.hcordova.flightagencyapi.agent.domain.repositories.AgentRepository;
import com.hcordova.flightagencyapi.passenger.domain.models.Passenger;
import com.hcordova.flightagencyapi.passenger.domain.repositories.PassengerRepository;
import com.hcordova.flightagencyapi.shared.domain.models.User;
import com.hcordova.flightagencyapi.shared.domain.repositories.RoleRepository;
import com.hcordova.flightagencyapi.shared.domain.repositories.UserRepository;
import com.hcordova.flightagencyapi.shared.domain.services.AuthenticationService;
import com.hcordova.flightagencyapi.shared.domain.services.JwtService;
import com.hcordova.flightagencyapi.shared.dto.SignInRequestDTO;
import com.hcordova.flightagencyapi.shared.dto.SignInResponseDTO;
import com.hcordova.flightagencyapi.shared.dto.SignUpRequestDTO;
import com.hcordova.flightagencyapi.shared.dto.SignUpResponseDTO;
import com.hcordova.flightagencyapi.shared.exceptions.NotFoundException;
import com.hcordova.flightagencyapi.supervisor.domain.models.Supervisor;
import com.hcordova.flightagencyapi.supervisor.domain.repositories.SupervisorRepository;

import jakarta.transaction.Transactional;
import lombok.RequiredArgsConstructor;

@Service
@RequiredArgsConstructor
public class AuthenticationServiceImpl implements AuthenticationService {

    private final UserRepository userRepository;
    private final PassengerRepository passengerRepository;
    private final AgentRepository agentRepository;
    private final SupervisorRepository supervisorRepository;
    private final RoleRepository roleRepository;
    private final JwtService jService;
    private final AuthenticationManager authenticationManager;
    private final PasswordEncoder passwordEncoder;

    @Override
    public ResponseEntity<SignInResponseDTO> signIn(SignInRequestDTO requestDTO) {
        authenticationManager.authenticate(
            new UsernamePasswordAuthenticationToken(requestDTO.getEmail(), requestDTO.getPassword()));
        var user = userRepository.findByEmail(requestDTO.getEmail())
            .orElseThrow(() -> new UsernameNotFoundException("Invalid email or password"));
        var jwt = jService.generateToken(user);

        var id = getIdByRoleAndUserId(user.getRole().getName(), user.getId());

        var response = new SignInResponseDTO(id, user.getName(), user.getLastname(), user.getEmail(), user.getRole().getName(), jwt);

        return ResponseEntity.status(HttpStatus.OK).body(response);
    }

    private Long getIdByRoleAndUserId(String role, Long id) {
        switch(role.toLowerCase()) {
            case "passenger":
                return passengerRepository.findByUserId(id).get().getId();
            case "agent":
                return agentRepository.findByUserId(id).get().getId();
            case "supervisor":
                return supervisorRepository.findByUserId(id).get().getId();
        }
        return 0L;
    }

    @Override
    @Transactional
    public ResponseEntity<SignUpResponseDTO> signUp(SignUpRequestDTO requestDTO) {
        var userFound = userRepository.findByEmail(requestDTO.getEmail());

        if (userFound.isPresent()) 
            throw new IllegalArgumentException("Email already exists!");

        var role = roleRepository.findByName(requestDTO.getRole().toLowerCase());

        if (role.isEmpty()) 
            throw new NotFoundException("Role not found!");

        var user = new User(
            requestDTO.getName(), 
            requestDTO.getLastname(), 
            requestDTO.getEmail(), 
            passwordEncoder.encode(requestDTO.getPassword()), 
            role.get());
        
        var userCreated = userRepository.save(user);
        Long userId = 0L;

        switch(userCreated.getRole().getName().toLowerCase()) {
            case "passenger":
                Passenger passenger = passengerRepository.save(buildPassenger(requestDTO, userCreated));
                userId = passenger.getId();
                break;
            case "agent":
                Agent agent = agentRepository.save(buildAgent(requestDTO, userCreated));
                userId = agent.getId();
                break;
            case "supervisor":
                Supervisor supervisor = supervisorRepository.save(buildSupervisor(requestDTO, userCreated));
                userId = supervisor.getId();
                break;
        }

        var response = new SignUpResponseDTO(userId, userCreated.getName(), userCreated.getLastname(), userCreated.getEmail(), userCreated.getRole().getName());
    
        return ResponseEntity.status(HttpStatus.CREATED).body(response);
    }

    private Passenger buildPassenger(SignUpRequestDTO requestDTO, User userCreated) {
        return new Passenger(
                0L,
                requestDTO.getCountry(), 
                requestDTO.getCity(),
                requestDTO.getAddress(),
                requestDTO.getPhone(),
                LocalDate.now(),
                null, 
                userCreated);
    }

    private Agent buildAgent(SignUpRequestDTO requestDTO, User userCreated) {
        return new Agent(
                0L,
                requestDTO.getDetails(),
                userCreated,
                LocalDate.now(),
                null);
    }

    private Supervisor buildSupervisor(SignUpRequestDTO requestDTO, User userCreated) {
        return new Supervisor(
                0L,
                requestDTO.getDetails(),
                LocalDate.now(),
                null,
                userCreated);
    }
    
}
