import { Component, OnInit } from '@angular/core';
import { Step } from 'src/app/shared/interfaces/step.interface';
import { TableModel } from 'src/app/shared/interfaces/table-model.interface';
import { PassengerService } from '../../services/passenger.service';
import { Passenger } from '../../interfaces/passenger.interface';
import { FormControl, FormGroup, Validators } from '@angular/forms';
import { Flight } from 'src/app/supervisor/interfaces/flight.interface';
import { TicketTypeService } from '../../services/ticket-type.service';
import { TicketClassService } from '../../services/ticket-class.service';
import { DropdownElement } from 'src/app/shared/interfaces/dropdown-element-interface';
import { Reservation } from '../../interfaces/reservation.interface';
import { ReservationsService } from '../../services/reservations.service';
import { Location } from '@angular/common';
import { ActivatedRoute } from '@angular/router';
import { FlightService } from 'src/app/supervisor/services/flight.service';
import { SecurityService } from 'src/app/security/services/security.service';
import { ReservationSave } from '../../interfaces/reservation-save.interface';

@Component({
  selector: 'app-reservation-add',
  templateUrl: './reservation-add.component.html',
  styleUrls: ['./reservation-add.component.css']
})
export class ReservationAddComponent implements OnInit{

  public steps: Step[] = [
    {
      id: 1,
      title: 'Información base',
      icon: 'bi-person',
      detail: "Información general de la reserva"
    },
    {
      id: 2,
      title: 'Selección de vuelo',
      icon: 'bi-airplane-fill',
      detail: "Información general de la reserva"
    },
    {
      id: 3,
      title: 'Resumen',
      icon: 'bi-check',
      detail: "Valida la configuración"
    }
  ];

  public flightModel: TableModel[] = [
    {
      name: 'id',
      title: '#'
    },
    {
      name: 'airline',
      title: 'Aerolinea'
    },
    {
      name: 'aircraft',
      title: 'Tipo de avión'
    },
    {
      name: 'flightType',
      title: 'Tipo de vuelo'
    },
    {
      name: 'departureDate',
      title: 'Hora de inicio'
    },
    {
      name: 'arrivalDate',
      title: 'Hora de llegada'
    },
    {
      name: 'price',
      title: 'Precio'
    }
  ];

  // Base Information Form

  public formBaseInformation: FormGroup = new FormGroup({
    ticketType: new FormControl<string | null>(null, [Validators.required]),
    flightClass: new FormControl<string | null>(null, [Validators.required]),
    passenger: new FormControl<string | null>(null, [Validators.required])
  });

  public formFlightInformation: FormGroup = new FormGroup({
    airportOrigin: new FormControl<number | null>(null, [Validators.required]),
    airportDestination: new FormControl<number | null>(null, [Validators.required]),
    date: new FormControl<string | null>(null, [Validators.required])
  });

  get passengerName(): string {
    if (this.formBaseInformation.value) {
      const passenger = this.passengers.find(passenger => passenger.id == this.formBaseInformation.value.passenger);
      return passenger ? `${passenger?.name} ${passenger?.lastname}` : "";
    }
    return "";
  }

  get ticketTypeName(): string {
    if (this.formBaseInformation.value) {
      const ticketType = this.ticketTypes.find(ticketType => ticketType.value == this.formBaseInformation.value.ticketType);
      return ticketType ? ticketType.text : "";
    }
    return "";
  }

  get flightClassName(): string {
    if (this.formBaseInformation.value) {
      const flightClass = this.flightClass.find(flightClass => flightClass.value == this.formBaseInformation.value.flightClass);
      return flightClass ? flightClass.text : "";
    }
    return "";
  }

  public passengers: Passenger[] = [];
  public ticketTypes: DropdownElement[] = [];
  public flightClass: DropdownElement[] = [];

  get isInvalidThisStep(): boolean {
    if (this.stepIndex == 0) return this.formBaseInformation.invalid;
    if (this.stepIndex == 1) return this.formFlightInformation.invalid || this.flightSelected === null;
    return false;
  }

  constructor(private passengersService: PassengerService,
    private ticketTypesService: TicketTypeService,
    private ticketClassService: TicketClassService,
    private reservationsService: ReservationsService,
    private flightService: FlightService,
    private location: Location,
    private activatedRoute: ActivatedRoute,
    private securityService: SecurityService) { }

  ngOnInit(): void {
    this.activatedRoute.params
      .subscribe(params => {
        if (params['id']) {
          this.reservationsService.getById(params['id'])
            .subscribe(reservation => {
              this.formBaseInformation.get('passenger')?.setValue(reservation.passengerId);
              this.formBaseInformation.get('ticketType')?.setValue(reservation.ticketTypeId);
              this.formBaseInformation.get('flightClass')?.setValue(reservation.travelClassId);
              this.formFlightInformation.get('airportOrigin')?.setValue(reservation.airportOriginId);
              this.formFlightInformation.get('airportDestination')?.setValue(reservation.airportDestinationId);
              this.formFlightInformation.get('date')?.setValue(reservation.departureDate);
              this.flightService.searchFlight(reservation.airportOriginId, reservation.airportDestinationId, reservation.departureDate)
                .subscribe(flights => {
                  // this.flightResults = flights;
                  this.flightSelected = flights.find(flight => flight.id === reservation.flightId) || null;
                })
            })
        }
        this.getAllData();
      })
  }

  getAllData(): void {
    this.passengersService.getAll()
    .subscribe(passengers => this.passengers = passengers);

    this.ticketTypesService.getAll()
    .subscribe(ticketTypes => {
      ticketTypes.map(ticketType => {
        this.ticketTypes.push({ text: ticketType.name, value: ticketType.id});
      });
    });

    this.ticketClassService.getAll()
      .subscribe(flightClass => {
        flightClass.map(flight => {
          this.flightClass.push({ text: flight.name, value: flight.id });
        })
    });
  }

  public flightSelected: Flight | null = null;

  public stepIndex: number = 0;

  nextStep(): void {
    if (this.stepIndex === this.steps.length - 1) {
      this.onSaveBooking();
      return;
    };
    this.stepIndex++;
  }

  prevStep(): void {
    if (this.stepIndex === 0) return;
    this.stepIndex--;
  }

  onChangeStep(step: number): void {
    this.stepIndex = step;
  }

  isFirstStep(): boolean {
    return this.stepIndex === 0;
  }

  isLastStep(): boolean {
    return this.stepIndex === this.steps.length - 1;
  }

  onSelectFlight(flight: Flight): void {
    this.flightSelected = flight;
  }

  onSaveBooking(): void {
    const reservation: ReservationSave = {
      agentId: this.securityService.currentUser?.id || 0,
      passengerId: this.formBaseInformation.value.passenger,
      travelClassId: parseInt(this.formBaseInformation.value.ticketType),
      ticketTypeId: parseInt(this.formBaseInformation.value.flightClass),
      reservationMade: this.formFlightInformation.value.date,
      flightId: this.flightSelected?.id || 0,
      cost: this.flightSelected?.price || 0
    };

    console.log(reservation);

    this.reservationsService.save(reservation)
      .subscribe(() => {
        this.formBaseInformation.reset();
        this.formFlightInformation.reset();
        this.flightSelected = null;
        this.location.back();
      })
  }
}
