import { Component, Input } from '@angular/core';
import { FormControl, FormGroup, Validators } from '@angular/forms';
import { TableModel } from 'src/app/shared/interfaces/table-model.interface';
import { FlightCost } from '../../interfaces/flight-cost.interface';

@Component({
  selector: 'supervisor-costs-config',
  templateUrl: './costs-config.component.html',
  styleUrls: ['./costs-config.component.css']
})
export class CostsConfigComponent {

  public costModel: TableModel[] = [
    {
      name: 'id',
      title: '#'
    },
    {
      name: 'dateStart',
      title: 'Fecha Inicio'
    },
    {
      name: 'dateEnd',
      title: 'Fecha Fin'
    },
    {
      name: 'cost',
      title: 'Costo (USD)'
    }
  ]

  @Input()
  public costs: FlightCost[] = [];

  public openDialogCost: boolean = false;

  public dateStartControl: FormControl = new FormControl<string | null>(null, [Validators.required]);
  public dateEndControl: FormControl = new FormControl<string | null>(null, [Validators.required]);
  public costControl: FormControl = new FormControl<number | null>(null, [Validators.required]);

  public costsForm: FormGroup = new FormGroup({
    dateStart: this.dateStartControl,
    dateEnd: this.dateEndControl,
    cost: this.costControl,
  }, {
    validators: () => {
      const dateStart = this.dateStartControl.value as string;
      const dateEnd = this.dateEndControl.value as string;
      const cost = this.costControl.value as number;

      this.crossValidationsDates(dateStart, dateEnd);
      this.crossValidationsCost(cost);
      return null;
    }
  });

  crossValidationsDates(dateStartString: string, dateEndString: string): void {
    const dateStart = new Date(dateStartString);
    const dateEnd = new Date(dateEndString);

    if (dateStartString == null) this.dateStartControl.setErrors({ required: true });
    else if (this.costs.filter(cost => {
      const dateStartCost = new Date(cost.dateStart);
      const dateEndCost = new Date(cost.dateEnd);
      return dateStart.getTime() >= dateStartCost.getTime() && dateStart.getTime() <= dateEndCost.getTime()
    }).length > 0) {
      this.dateStartControl.setErrors({ existCost: true });
    }

    if (dateEndString == null) this.dateEndControl.setErrors({ required: true });
    else if (dateStart.getTime() >= dateEnd.getTime()) {
      this.dateEndControl.setErrors({ isInvalid: true });
    } else this.dateEndControl.setErrors(null);
  }

  crossValidationsCost(cost: number): void {
    if (cost === null) this.costControl.setErrors({ required: true });
    else if (cost == 0) this.costControl.setErrors({ numberZero: true });
    else if (cost < 0) this.costControl.setErrors({ numberNegative: true });
    else this.costControl.setErrors(null);
  }

  onTogleDialogCost(): void {
    this.openDialogCost = !this.openDialogCost;
    this.costsForm.reset();
  }

  onSaveCost(): void {
    this.costsForm.markAllAsTouched();
    if (this.costControl.valid) {
      let cost: FlightCost = {
        id: this.costs.length + 1,
        dateStart: this.dateStartControl.value as string,
        dateEnd: this.dateEndControl.value as string,
        cost: parseInt(this.costControl.value)
      }
      this.costs.push(cost);
      this.onTogleDialogCost();
    }
  }
}
