import { Component, Input, OnInit } from '@angular/core';
import { FormControl, FormGroup, Validators } from '@angular/forms';
import { TableModel } from 'src/app/shared/interfaces/table-model.interface';
import { FlightScale } from '../../interfaces/flight-scale.interface';
import { DropdownElement } from 'src/app/shared/interfaces/dropdown-element-interface';
import { AirportsService } from 'src/app/agent/services/airports.service';
import { FlightTable } from '../../interfaces/flight-table.interface';

@Component({
  selector: 'supervisor-scales-config',
  templateUrl: './scales-config.component.html',
  styleUrls: ['./scales-config.component.css']
})
export class ScalesConfigComponent implements OnInit {

  constructor(private airportsService: AirportsService) { }

  ngOnInit(): void {
    this.getAllAirports();
  }

  public scaleModel: TableModel[] = [
    {
      name: 'id',
      title: '#'
    },
    {
      name: 'airportOrigin',
      title: 'Origen'
    },
    {
      name: 'airportDestination',
      title: 'Destino'
    },
    {
      name: 'departureDateTime',
      title: 'Hora Inicio'
    },
    {
      name: 'arrivalDateTime',
      title: 'Hora Fin'
    }
  ]

  public openDialogScale: boolean = false;

  @Input()
  public scales: FlightScale[] = [];

  public airports: DropdownElement[] = [];

  public airportOriginIdControl: FormControl<number | null> = new FormControl(null, [Validators.required]);
  public airportDestinationIdControl: FormControl<number | null> = new FormControl(null, [Validators.required]);
  public departureDateTimeControl: FormControl<string | null> = new FormControl(null, [Validators.required]);
  public arrivalDateTimeControl: FormControl<string | null> = new FormControl(null, [Validators.required]);

  public scalesForm: FormGroup = new FormGroup({
    airportOriginId: this.airportOriginIdControl,
    airportDestinationId: this.airportDestinationIdControl,
    departureDateTime: this.departureDateTimeControl,
    arrivalDateTime: this.arrivalDateTimeControl,
  }, {
    validators: () => {
      const airportOriginId = this.airportOriginIdControl.value;
      const airportDestinationId = this.airportDestinationIdControl.value;
      const departureDate = this.departureDateTimeControl.value;
      const arrivalDate = this.arrivalDateTimeControl.value;

      return this.scaleCrossValidation(airportOriginId, airportDestinationId, departureDate, arrivalDate);
    }
  });

  scaleCrossValidation(airportOriginId: number | null, airportDestinationId: number | null,
    departureDate: string | null, arrivalDate: string | null): { [key: string]: boolean } | null {
      if (airportDestinationId == null) this.airportDestinationIdControl.setErrors({ required: true });
      else if (airportOriginId === airportDestinationId) {
        this.airportDestinationIdControl.setErrors({ airportEqual: true });
      } else {
        this.airportDestinationIdControl.setErrors(null);
      }

      if (departureDate == null) this.departureDateTimeControl.setErrors({ required: true });
      else if (this.scales.filter(scale => {
        const scaleDepartureDate = new Date(scale.departureDateTime);
        const scaleArrivalDate = new Date(scale.arrivalDateTime);
        const departureDateCompare = new Date(departureDate);

        return (departureDateCompare >= scaleDepartureDate && departureDateCompare <= scaleDepartureDate) ||
          (departureDateCompare >= scaleArrivalDate && departureDateCompare <= scaleArrivalDate)
      }).length > 0) {
        this.departureDateTimeControl.setErrors({ existCost: true });
      } else this.departureDateTimeControl.setErrors(null);

      if (arrivalDate == null) this.arrivalDateTimeControl.setErrors({ required: true });
      else if (departureDate! >= arrivalDate) this.arrivalDateTimeControl.setErrors({ isInvalid: true });
      else this.arrivalDateTimeControl.setErrors(null);

      return null;
  }

  onTogleDialogScale(): void {
    this.openDialogScale = !this.openDialogScale;
    this.scalesForm.reset();
    this.scales.length > 0 ? this.airportOriginIdControl.setValue(this.scales[this.scales.length - 1].airportDestinationId) : this.airportOriginIdControl.setValue(null);
  }

  getAllAirports(): void {
    this.airportsService.getAll()
    .subscribe(airports =>
      { this.airports = airports.map( airport => { return { value: airport.id, text: airport.name } } )});
  }

  getAirportById(id: number): string {
    return this.airports.find(airport => airport.value == id)?.text || '';
  }

  onSaveScale(): void {
    this.scalesForm.markAllAsTouched();
    if (this.scalesForm.valid) {
      const scale: FlightScale = {
        id: this.scales.length + 1,
        airportOrigin: this.getAirportById(this.airportOriginIdControl.value!),
        airportOriginId: parseInt(this.scalesForm.value.airportOriginId),
        airportDestination: this.getAirportById(this.airportDestinationIdControl.value!),
        airportDestinationId: parseInt(this.scalesForm.value.airportDestinationId),
        departureDateTime: this.scalesForm.value.departureDateTime,
        arrivalDateTime: this.scalesForm.value.arrivalDateTime
      }
      this.scales.push(scale);
      this.onTogleDialogScale();
    }
  }
}
